// =====================================================================================
//
//       Filename:  sql_lib.h
//
//    Description:  
//                     code edit with vim and set tw=2, default code is under utf-8.
//
//        Version:  $Id$
//        Created:  2020/9/8 10:26:34
//       Revision:  none
//       Compiler:  modern c compiler, test under vs2019.
//
//         Author:  lk (dd), lk@shucantech.com
//				Company:  SCT
//   Organization:  SCT-P
//
//				License:  SCT-CL V0
//
// =====================================================================================


#pragma once
#include <stdint.h>
#include <stdbool.h>

#define _CRT_SECURE_NO_WARNINGS

#define   VAL_SIZE_1 1	//VALUE SIZE 1
#define   VAL_SIZE_2 2	//VALUE SIZE 2
#define   VAL_SIZE_4 4	//VALUE SIZE 4
#define   VAL_SIZE_8 8	//VALUE SIZE 8
#define   VAL_SIGN   0	//VALUE SIGNED
#define   VAL_USGN   1	//VALUE UNSIGNED

#define   VAL_TYPE_INV -1	//VALUE TYPE INVALID
#define   VAL_TYPE_I 0		//VALUE TYPE INTEGER
#define   VAL_TYPE_F 1		//VALUE TYPE FLOAT
#define   VAL_TYPE_S 2		//VALUE TYPE STRING
#define   VAL_TYPE_D 3		//VALUE TYPE DATE
#define   VAL_TYPE_T 4		//VALUE TYPE TIME
#define   VAL_TYPE_DT 5		//VALUE TYPE DATETIME

#ifdef _WIN32
#ifdef SQL_LIB_EXPORT
#define dllemport __declspec(dllexport)
#else
#define dllemport __declspec(dllimport)
#endif
#else
#define dllemport
#endif

#ifdef __cplusplus
extern "C" {
#endif

// ===  FUNCTION  ======================================================================
//         Name:  connect_to_server
//  Description:  连接到服务端
//								参数ip:		服务端ip地址
//								参数port: 服务端监听端口
//								参数user: 服务端验证用户名
//								参数pwd:	服务端验证密码
//
//								返回值:   若服务端连接成功并通过验证则返回true, 否则返回false。
// =====================================================================================
dllemport bool connect_to_server(const char* ip, int port, const char* user, const char* pwd);


// ===  FUNCTION  ======================================================================
//         Name:  exit_to_server
//  Description:  从服务端退出
//
//								返回值:   若服务端存在连接并退出成功返回true, 否则返回false。
// =====================================================================================
dllemport bool exit_to_server();

// ===  FUNCTION  ======================================================================
//         Name:  exec
//  Description:  在服务端执行sql语句
//
//								参数sql:	服务端需要执行的sql语句
//                参数c  :  sql语句大小
//								返回值:   若服务端执行成功返回有效的结果集指针, 否则返回NULL指针
// =====================================================================================
dllemport void* exec(const char* sql, int c);

// ===  FUNCTION  ======================================================================
//         Name:  del_datas
//  Description:  删除执行的返回的结果集
//
//								参数d:	  服务端执行结果集指针
// =====================================================================================
dllemport void del_datas(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  rows_of
//  Description:  获取结果集的数据行数
//
//								参数d:	  服务端执行结果集指针
//								返回值:   若结果集为空返回0, 否则返回结果行数
// =====================================================================================
dllemport uint64_t rows_of(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  cols_of
//  Description:  获取结果集的数据列数
//
//								参数d:	  服务端执行结果集指针
//								返回值:   若结果集为空返回0, 否则返回结果列数
// =====================================================================================
dllemport uint64_t cols_of(void* d);

// ===  FUNCTION  ======================================================================
//         Name:  sid_of
//  Description:  获取结果集的对应行的内部系统id(同id()函数)
//
//								参数d:	  服务端执行结果集指针
//								参数r:	  结果集行号
//								返回值:   返回结果集系统id, SIZEMAX意味该id为生成数据不可编辑
// =====================================================================================
dllemport uint64_t sid_of(void* d, uint64_t r);

// ===  FUNCTION  ======================================================================
//         Name:  col_name
//  Description:  获取结果集某一列列名
//
//								参数col_sz:	  结果集对应列的长度指针
//								参数d:				服务端执行结果集指针
//								参数col_idx:  结果集的列索引(从0开始)
//								返回值:				若结果集为空指针或列数为0返回NULL, 否则返回列名常指针
//															col_sz中存入列名长度, 注意:若d被删除则返回地址为非法指针
// =====================================================================================
dllemport const char* col_name(int* col_sz, void* d, int col_idx);

// ===  FUNCTION  ======================================================================
//         Name:  at
//  Description:  获取结果集某一行列的值
//
//								参数row:	  结果集行索引(从0开始)
//								参数col:		结果集列索引(从0开始)
//								参数d:			服务端执行的结果集指针
//								返回值:			若结果集为空指针或行/列索引无效, 返回NULL, 
//														否则返回对应位置值指针, 注意:若d被删除则返回地址为非法指针
// =====================================================================================
dllemport void* at(uint64_t row, uint64_t col, void* d);

// ===  FUNCTION  ======================================================================
//         Name:  print
//  Description:  控制台输出结果集
//
//								参数d:			服务端执行的结果集指针
// =====================================================================================
dllemport void print(void* d);

// ===  FUNCTION  ======================================================================
//         Name:	val_size
//  Description:  获取结果值字节大小
//  
//								参数v:			值指针
//								返回值:			若值为空或字符串类型返回NULL,	否则返回对应值大小(字节单位) 
//														字节大小定义为VAL_SIZE_X
// =====================================================================================
dllemport int val_size(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	val_type
//  Description:  获取结果值类型
//  
//								参数v:			值指针
//								返回值:			若值为空返回VAL_TYPE_INV,	否则返回对应值类型 
//														字节大小定义为VAL_TYPE_X
// =====================================================================================
dllemport int val_type(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	val_sign
//  Description:  获取结果值符号性
//  
//								参数v:			值指针
//								返回值:			若值为整数数值有符号型返回VAL_SIGN,	否则返回VAL_USGN
//														值为空则行为异常
// =====================================================================================
dllemport int val_sign(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_int8_t
//  Description:  转换值指针为int8类型
//  
//								参数v:			值指针
//								返回值:			若值为int8则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport int8_t as_int8_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_int16_t
//  Description:  转换值指针为int16类型
//  
//								参数v:			值指针
//								返回值:			若值为int16则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport int16_t as_int16_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_int32_t
//  Description:  转换值指针为int32类型
//  
//								参数v:			值指针
//								返回值:			若值为int32则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport int32_t as_int32_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_int64_t
//  Description:  转换值指针为int64类型
//  
//								参数v:			值指针
//								返回值:			若值为int64则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport int64_t as_int64_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_uint8_t
//  Description:  转换值指针为uint8类型
//  
//								参数v:			值指针
//								返回值:			若值为uint8则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport uint8_t as_uint8_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_uint16_t
//  Description:  转换值指针为uint16类型
//  
//								参数v:			值指针
//								返回值:			若值为uint16则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport uint16_t as_uint16_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_uint32_t
//  Description:  转换值指针为uint32类型
//  
//								参数v:			值指针
//								返回值:			若值为intu32则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport uint32_t as_uint32_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_uint64_t
//  Description:  转换值指针为uint64类型
//  
//								参数v:			值指针
//								返回值:			若值为uint64则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport uint64_t as_uint64_t(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_float
//  Description:  转换值指针为float类型
//  
//								参数v:			值指针
//								返回值:			若值为float则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport float as_float(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_double
//  Description:  转换值指针为double类型
//  
//								参数v:			值指针
//								返回值:			若值为double则转换为对应值类型, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport double as_double(void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_str
//  Description:  转换值指针为string类型
//  
//								参数b:			字符串指针地址
//								参数c:			字符串长度指针
//								参数v:			值指针
//								返回值:			若值为string则转换为对应值类型存储于*b, 否则作内存强制转换
//														值为空则行为异常
// =====================================================================================
dllemport void as_str(char** b, uint64_t* c, void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_date
//  Description:  转换值指针为date类型
//  
//								参数y:			年数指针
//								参数m:			月数指针
//								参数d:			天数指针
//								参数v:			值指针
//								返回值:			若值为date则存储值于y, m, d, 否则行为异常
// =====================================================================================
dllemport void as_date(int* y, int* m, int* d, void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_time
//  Description:  转换值指针为time类型
//  
//								参数h:			小时数指针
//								参数m:			分钟数指针
//								参数s:			秒数指针
//								参数n:			纳秒数指针
//								参数v:			值指针
//								返回值:			若值为date则存储值于h, m, s, n, 否则行为异常
// =====================================================================================
dllemport void as_time(int* h, int* m, int* s, int* n, void* v);

// ===  FUNCTION  ======================================================================
//         Name:	as_datetime
//  Description:  转换值指针为time类型
//  
//								参数y:			年数指针
//								参数m:			月数指针
//								参数d:			天数指针
//								参数h:			小时数指针
//								参数mi:			分钟数指针
//								参数s:			秒数指针
//								参数n:			纳秒数指针
//								参数v:			值指针
//								返回值:			若值为date则存储值于h, m, s, n, 否则行为异常
// =====================================================================================
dllemport void as_datetime(int* y, int* m, int* d, int* h, int* mi, int* s, int* n, void* v);

#ifdef __cplusplus
}
#endif
